<?php
/**
 * Created by PhpStorm.
 * User: max18
 * Date: 30.04.2020
 * Time: 11:16
 */

use \MM\bot\controller\BotController;
use \MM\bot\components\standard\Text;

class NeverController extends BotController
{
    protected const CHILDREN_MODE = 0; // Детский режим
    protected const NORMAL_MODE = 1; // Нормальный режим
    protected const HARD_MODE = 2; // Режим, в котором объединены детский и нормальный режим

    protected const SEX_MODE = 3; // Режим 18+

    protected const DEFAULT_MODE = 0; // Стандартный режим

    /**
     * Доступные режимы
     */
    protected const MODES = [
        self::CHILDREN_MODE => [
            'name' => 'Детский режим', // Название режима
            'img' => '937455/038af42397fb33d3a91a', // Токен изображения
            'desc' => 'Вопросы и действия, предназначенные для детей', // Описание режима
            'actions' => ['дете', 'детски', 'легки'] // Фразы, по которым происходит понимание того, что выбран этот режим
        ],
        self::NORMAL_MODE => [
            'name' => 'Стандартный режим',
            'img' => '1030494/5f1705d1c43609428bda',
            'desc' => 'Вопросы и действия, предназначенные для взрослых',
            'actions' => ['просто', 'стандарт', 'норм']
        ],
        self::HARD_MODE => [
            'name' => 'Сложный режим',
            'img' => '1540737/d5fa4f76a13aac898427',
            'desc' => 'Вопросы и действия как для взрослых, так и для детей',
            'actions' => ['сложнее', 'сложны', 'проф']
        ],
        self::SEX_MODE => [
            'name' => 'Для взрослых 18+',
            'img' => '213044/fad3f97927d18a76ad3b',
            'desc' => 'Вопросы исключительно 18+',
            'actions' => ['18', 'взрослы', 'эротич'],
            'isAuth' => true
        ]
    ];

    /**
     * Получение списка вопросов, для нужного режима
     * @param string $fileName Название файла. Режим + .json
     * @return mixed
     */
    private function getFileContent(string $fileName)
    {
        return json_decode(file_get_contents(__DIR__ . "/question/{$fileName}"), true);
    }

    /**
     * Получение списка всех вопросов из категории
     * @return array|mixed
     */
    protected function getQuestionsList()
    {
        /**
         * Если режим не указан, то используем режим по умолчанию
         */
        $mode = $this->userData['mode'] ?? self::DEFAULT_MODE;
        $content = [];
        switch ($mode) {
            case self::CHILDREN_MODE:
                $content = $this->getFileContent(self::CHILDREN_MODE . '.json');
                break;
            case self::NORMAL_MODE:
                $content = $this->getFileContent(self::NORMAL_MODE . '.json');
                break;
            case self::HARD_MODE:
                /**
                 * Для сложного режима объединяем вопросы из детского и стандартного режима.
                 */
                $arr1 = $this->getFileContent(self::CHILDREN_MODE . '.json');
                $arr2 = $this->getFileContent(self::NORMAL_MODE . '.json');
                $cArr1 = count($arr1);
                $cArr2 = count($arr2);
                if ($cArr2 > $cArr1) {
                    for ($i = 0; $i < $cArr2; $i++) {
                        $content[] = $arr2[$i];
                        if (isset($arr1[$i])) {
                            $content[] = $arr1[$i];
                        }
                    }
                } else {
                    for ($i = 0; $i < $cArr1; $i++) {
                        $content[] = $arr1[$i];
                        if (isset($arr2[$i])) {
                            $content[] = $arr2[$i];
                        }
                    }
                }
                break;

            case self::SEX_MODE:
                $content = $this->getFileContent(self::SEX_MODE . '.json');
                break;
        }
        return $content;
    }

    /**
     * Получение 1 случайного вопроса. Чтобы не использовать рандом постоянно, используем его только при 1 заходе, а после просто отображаем вопросы по порядку. В таком случае, мы избавляемся от момента, когда при рандоме, один и тотже вопрос может задаться несколько раз.
     * @param bool $isNew
     * @return mixed
     */
    protected function getQuestion($isNew = false)
    {
        $list = $this->getQuestionsList();
        $maxCount = count($list);
        /**
         * Если ушли за пределы массива вопросов, или начинаем новую игру, тогда выбираем случайный вопрос.
         */
        if (!isset($this->userData['index']) || $isNew) {
            $this->userData['index'] = rand(0, $maxCount - 1) - 1;
            if ($this->userData['index'] < 0) {
                $this->userData['index'] = -1;
            }
        }

        $this->userData['index']++;
        if ($this->userData['index'] >= $maxCount) {
            $this->userData['index'] = 0;
        }

        $text = $list[$this->userData['index']];
        /**
         * Записываем вопрос, чтобы можно было воспроизвести его еще раз, если пользователь его не расслышал
         */
        $this->userData['text'] = $text;
        return $text;
    }

    /**
     * Начинаем игру
     */
    protected function game()
    {
        /**
         * Если пользователь только запустил навык, тогда ставим шаг в 0.
         * Шаг нужен для того, чтобы понять в каком месте находится пользователь.
         */
        $this->userData['step'] = $this->userData['step'] ?? 0;
        $this->thisIntentName = 'game';
        switch ($this->userData['step']) {
            case 0:
                /**
                 * Спрашиваем, готов ли пользователь начать игру + отображаем карточку для интерактивности
                 */
                $this->tts = "Отлично!\nГотов начать игру?";
                $this->card->title = $this->tts;
                $this->card->add('997614/743bc730adb717da13d7', 'Да', '', 'Да');
                $this->card->add('213044/31a85be9652f407e4af3', 'Нет', '', 'Нет');
                $this->userData['step']++;
                break;
            case 1:
                /**
                 * Проверяем что пользователь выражает согласие/не согласие.
                 * И в зависимости от ответа, начинаем игру или нет.
                 * Если пользователь говорит что-то непонятно, то говорим что не понимаем его, и просим ответить либо да, либо нет
                 */
                if ($this->nlu->isIntentConfirm($this->userCommand)) {
                    $this->tts = "Отлично!\nТогда начинаем!";
                    $neverText = $this->getQuestion(true);
                    $this->card->title = $this->tts;
                    $this->card->add($this->getNewerImg(), 'Я никогда не', $neverText);
                    $this->card->add($this->getNextImg(), 'Дальше', '', 'Дальше');
                    $this->tts .= "Я никогда не {$neverText}";
                    $this->userData['step']++;
                } elseif ($this->nlu->isIntentReject($this->userCommand)) {
                    $this->text = 'Нет, так нет. Поиграем в другой раз.';
                    $this->card->title = $this->text;

                    /**
                     * Отображаем большую картинку пользователю
                     */
                    $this->card->add('1652229/5439806afcb47ebdbf19', $this->text, '');
                    $this->card->isOne = true;

                    unset($this->userData['step']);
                } else {
                    $this->tts = "Прости, но я тебя не понимаю...\nСкажи, ты готов начать игру?";
                    $this->card->title = $this->tts;
                    $this->card->add('997614/743bc730adb717da13d7', 'Да', '', 'Да');
                    $this->card->add('213044/31a85be9652f407e4af3', 'Нет', '', 'Нет');
                }
                break;
            case 2:
                /**
                 * Непосредственно игра и отображение вопросов
                 */
                $this->tts = $this->getNewerText();
                $neverText = $this->getQuestion();
                $this->card->title = $this->tts;
                $this->card->add($this->getNewerImg(), 'Я никогда не', $neverText);
                $this->card->add($this->getNextImg(), 'Дальше', '', 'Дальше');
                $this->tts .= "Я никогда не {$neverText}. \n{$this->getOptionText()}";
                break;
            default:
                break;
        }
    }

    /**
     * Текст который произносит навык перед вопросом.
     * Нужен чтобы пользователю не стало скучно от простых "Я никогда не ..."
     * @return string
     */
    protected function getNewerText()
    {
        $texts = [
            'Идем дальше!',
            'Следующее утверждение!',
            'А если так!',
            'Попробуем это!',
            'А как тебе такое?',
            'Далее',
            'А как тебе такой вариант?',
            'Ну ка!',
            'Что там у нас дальше?',
            'А что с этим?',
            'Ну а так!',
        ];
        return Text::getText($texts);
    }

    /**
     * После каждого вопроса, говорим пользователю что делать дальше.
     * Вроде бы не нужно, но с этим интереснее
     * @return string
     */
    protected function getOptionText()
    {
        $nextTexts = [
            'Для продолжения скажи "Дальше".',
            'Чтобы получить следующее утверждение, скажи "дальше"',
            'Чтобы продолжить скажи "Дальше"',
        ];
        $options = [
            'Если ты это делал, то загибай один палец.',
            'Делал это когда-то? Тогда загибай палец!',
            'Было? Тогда палец загни!',
        ];
        return Text::getText($options) . ' ' . Text::getText($nextTexts);
    }

    /**
     * Когда задаем вопрос пользователю, отображаем картинку, чтобы пользователь не видел одну и туже картинку, отображаем рандомную
     * @return string
     */
    protected function getNewerImg()
    {
        $images = [
            '965417/0f1d722f7a3d9fb8fc7f',
            '1030494/b5a33d872085da5f79f6',
            '965417/947fe606f9c004e0a823',
        ];
        return Text::getText($images);
    }

    /**
     * Добавление своего вопроса в навык.
     * @param $text
     */
    protected function addNever($text)
    {
        $fileName = $this->userData['mode'] . '.json';
        $list = $this->getFileContent($fileName);
        $list[] = $text;
        @file_put_contents(__DIR__ . "/question/{$fileName}", json_encode($list, JSON_UNESCAPED_UNICODE));
    }

    /**
     * Выбор режима игры
     */
    protected function mode()
    {
        /**
         * Говорим о том, что выбираем режим, и при последующем запросе, получим oldIntentName с mode
         * А также определяем на каком шаге выбора мы находимся
         */
        $this->thisIntentName = 'mode';
        $this->userData['step'] = $this->userData['step'] ?? 0;
        switch ($this->userData['step']) {
            case 0:
                $this->tts = 'Выбирай нужный режим и начинаем игру!';
                $this->card->title = $this->tts;
                $this->tts .= ' Доступны следующие режимы: ';
                /**
                 * Отображаем в виде карточки доступные режимы игры + проговариваем их
                 */
                foreach (self::MODES as $mode) {
                    $this->tts .= "- {$mode['name']} ;";
                    $this->card->add($mode['img'] ?? '', $mode['name'], '', $mode['name']);
                }
                $this->userData['step']++;
                break;
            case 1:
                $selectMode = null;
                foreach (self::MODES as $index => $mode) {
                    /**
                     * Определяем какой режим выбрал пользователь.
                     * И если это режим 18+ то задаем дополнительный вопрос про возраст, иначе начинаем игру.
                     */
                    if (Text::isSayText($mode['actions'], $this->userCommand)) {
                        if ($index == self::SEX_MODE) {
                            $this->tts = "Ты выбрал \"{$mode['name']}\". Скажи, тебе точно есть 18 лет?";
                            $this->card->title = 'Тебе точно 18 лет?';
                            $this->card->add('997614/743bc730adb717da13d7', 'Да', '', 'Да');
                            $this->card->add('213044/31a85be9652f407e4af3', 'Нет', '', 'Нет');
                            $this->userData['step'] = 2;
                            return;
                        } else {
                            $this->userData['mode'] = $index;
                            $this->tts = "Отлично! Ты выбрал \"{$mode['name']}\". Готов начать игру?";
                            $this->card->title = 'Готов начать игру?';
                            $this->card->add('997614/743bc730adb717da13d7', 'Да', '', 'Да');
                            $this->card->add('213044/31a85be9652f407e4af3', 'Нет', '', 'Нет');
                            $this->thisIntentName = 'game';
                            $this->userData['step'] = 1;
                            return;
                        }
                    }
                }
                /**
                 * Если не поняли какой режим, то заново отображаем все режимы и пытаемся понять что нужно.
                 */
                $this->tts = "Прости, но я тебя не понимаю...\nПожалуйста выбери один из доступных режимов:";
                $this->card->title = $this->tts;
                foreach (self::MODES as $mode) {
                    $this->tts .= "- {$mode['name']} ;";
                    $this->card->add($mode['img'] ?? '', $mode['name'], '', $mode['name']);
                }
                break;
            case 2:
                /**
                 * Попадаем сюда в том случае, если пользователь выбираем режим 18+
                 */
                if ($this->nlu->isIntentConfirm($this->userCommand)) {
                    $this->userData['mode'] = self::SEX_MODE;
                    $mode = self::MODES[self::SEX_MODE];
                    $this->tts = "Отлично! Ты выбрал \"{$mode['name']}\". Готов начать игру?";
                    $this->card->title = 'Готов начать игру?';
                    $this->card->add('997614/743bc730adb717da13d7', 'Да', '', 'Да');
                    $this->card->add('213044/31a85be9652f407e4af3', 'Нет', '', 'Нет');
                    $this->thisIntentName = 'game';
                    $this->userData['step'] = 1;
                } else {
                    $this->tts = "Выбери один из доступных режимов:";
                    $this->card->title = $this->tts;
                    foreach (self::MODES as $mode) {
                        $this->tts .= "- {$mode['name']} ;";
                        $this->card->add($mode['img'] ?? '', $mode['name'], '', $mode['name']);
                    }
                }
                return;
                break;
            default:
                /**
                 * Когда что-то пошло не так заново просим пользователя выбрать режим.
                 */
                $this->tts = 'Выбирай нужный режим и начинаем игру!';
                $this->card->title = $this->tts;
                $this->tts .= ' Доступны следующие режимы: ';
                foreach (self::MODES as $mode) {
                    $this->tts .= "- {$mode['name']} ;";
                    $this->card->add($mode['img'] ?? '', $mode['name'], '', $mode['name']);
                }
                $this->userData['step'] = 1;
                break;
        }
    }

    /**
     * Предыдущая команда пользователя. Если не удалось найти команду, то возвращаем false
     * @return bool
     */
    protected function processingOldCommand()
    {
        $status = true;
        switch ($this->oldIntentName) {
            case 'mode':
                $this->mode();
                break;
            case 'game':
                $this->game();
                break;
            default:
                $status = false;
                break;
        }
        return $status;
    }

    /**
     * Чтобы пользователю было интереснее, отображаем разную кнопку "дальше"
     * @return string
     */
    private function getNextImg()
    {
        $images = [
            '965417/ebc91915e2ccc9b0e96c',
            '213044/360f4e551e1a98841745',
            '997614/650d18068feefdf2c8e1'
        ];
        return Text::getText($images);
    }

    /**
     * Обработка команд и запросов пользователя
     * @param string|null $intentName Название интента, которое удалось определить.
     */
    public function action($intentName): void
    {
        if ($this->userData == null) {
            $this->userData = [];
        }

        switch ($intentName) {
            case WELCOME_INTENT_NAME:
                /**
                 * Приветствие пользователя.
                 * Говорим какой режим выбран и спрашиваем хочет пользователь начать игру или нет.
                 */
                $this->tts = 'Привет! Готов начать игру или хочешь выбрать режим игры?';
                $this->card->title = $this->tts;
                $modeIndex = $this->userData['mode'] ?? 0;
                $mode = self::MODES[$modeIndex]['name'];
                $this->tts .= "Сейчас выбран \"{$mode}\"!";
                $this->card->add('997614/743bc730adb717da13d7', 'Да начинаем игру', '', 'Да начинаем игру');
                $this->card->add('1030494/a4da1e3a4eb517387652', 'Выбрать режим игры', '', 'Выбрать режим игры');
                $this->userData['step'] = 1;
                $this->oldIntentName = 'game';
                break;

            case HELP_INTENT_NAME:
            case 'hint':
            case 'm_help':
                /**
                 * Отображаем текст помощи.
                 * Если мы находимся в режиме выбора режима игры, то рассказываем о режимах, и предлагаем выбрать нужный.
                 */
                if ($this->oldIntentName === 'mode') {
                    $this->tts = 'Выбери нужный режим, в котором хочешь начать игру.';
                    $this->card->title = $this->tts;
                    $this->tts .= ' Доступны следующие режимы: ';
                    foreach (self::MODES as $mode) {
                        $this->tts .= "- {$mode['name']} ;";
                        $this->card->add($mode['img'] ?? '', $mode['name'], '', $mode['name']);
                    }
                }
                /**
                 * Если пользователю не понимает что нужно делать, то подсказываем ему, и повторяем вопрос
                 */
                if ($intentName === 'hint') {
                    if ($this->oldIntentName === 'game') {
                        $this->tts = "Если ты когда-либо {$this->userData['text']}, тогда ты загибаешь один палец. Если ты уже загнул все пальцы, то просто начинай игру сначала 😁";
                        $this->card->title = $this->tts;
                        $this->card->add($this->getNewerImg(), 'Я никогда не', $this->userData['text']);
                        $this->card->add($this->getNextImg(), 'Дальше', '', 'Дальше');
                    }
                }
                /**
                 * Если ни одно из условий выше не сработало, тогда отображаем текст с подсказкой
                 */
                if (!$this->tts) {
                    $this->text = 'В чем заключается суть игры «Я никогда не»?
Правила игры довольно просты, а суть в том, чтобы отлично повеселиться.
В эту игру желательно играть минимум 4-5 человек, но можно и вдвоем.
Вы садитесь в круг, чтобы было комфортно общаться.
После чего вы выбираете нужную категорию игры, и начинаете играть. Алиса будет говорить то, что она никогда не делала в жизни.
Кто из игроков делал это, то он получает один штрафной балл.
Кто не делал этого, ничего не получает.
Можно играть до десяти, а штрафные баллы загибать пальцами. Либо использовать спички или монетки.
Победителем игры будет тот, у кого окажется меньше всего загнутых пальцев или штрафных баллов.
Секрет игры в выборе провокационных вопросов, которые помогут узнать других лучше и заставить получить лишний штраф.
Главное в игре быть предельно честным, а иначе смысл игры пропадает.';
                }
                break;

            case 'mode':
                /**
                 * Выбор режима игры
                 */
                if ($this->oldIntentName != 'mode') {
                    unset($this->userData['step']);
                    $this->mode();
                } else {
                    $this->processingOldCommand();
                }
                break;

            case 'start':
                /**
                 * Начало игры
                 */
                $this->userData['step'] = 0;
                $this->game();
                break;

            case 'next':
                /**
                 * Получение следующего вопроса
                 */
                $this->game();
                break;

            case 'replay':
                /**
                 * Повторяем вопрос пользователю
                 */
                $this->tts = "Повторяю еще раз!";
                $this->card->title = $this->text;
                $this->card->add($this->getNewerImg(), 'Я никогда не', $this->userData['text']);
                $this->card->add($this->getNextImg(), 'Дальше', '', 'Дальше');
                $this->tts .= "Я никогда не {$this->userData['text']}. \n{$this->getOptionText()}";
                break;

            case 'never':
                /**
                 * Когда пользователь начинает задавать вопрос Навыку
                 */
                $this->tts = 'Извини, но мы играем в мою игру, в данном случае, говорю я. 😁 И так!';
                $neverText = $this->getQuestion();
                $this->card->title = $this->tts;
                $this->card->add($this->getNewerImg(), 'Я никогда не', $neverText);
                $this->card->add($this->getNextImg(), 'Дальше', '', 'Дальше');
                $this->tts .= "Я никогда не {$neverText}. \n{$this->getOptionText()}";
                //$this->game();
                /**
                 * Сохраняем предложенный пользователем вопрос.
                 * Как показала практика, это плохая идея, так как пользователь предлагает много бреда...
                 */
                /*
                $this->tts = "Интересное предложение. На него я конечно не дам ответ, но спрошу тебя!";
                $this->card->title = 'Давай проверим!';
                $this->card->add($this->getNewerImg(), $this->originalUserCommand);
                $this->card->add($this->getNextImg(), 'Дальше', '', 'Дальше');
                $this->tts .= $this->originalUserCommand;
                $this->userData['text'] = $this->originalUserCommand;
                $this->addNever(trim(str_replace(['Я никогда не ', 'я никогда не '], '', $this->originalUserCommand)));
                */
                break;

            case 'by':
                /**
                 * Пользователь уходит из навыка
                 */
                $this->text = Text::getText(['Как жалко, что ты уходишь...', 'Возвращайся скорее!', 'Пока пока!']);
                $this->card->title = $this->text;
                $this->card->add(Text::getText(['1540737/d1124b93d0144cba37fc', '997614/0f027758444c36cefff0', '1521359/9ed57d853b16821a9381']), $this->text, '');
                $this->card->isOne = true;
                /**
                 * Говорим движку о том, что пользователь хочет покинуть навыкю
                 */
                $this->isEnd = true;
                break;

            default:
                /**
                 * Когда не удалось определить интент, то определяем контекст по oldIntentName. Если идентификатор предыдущей команды отсутствует, то предлагаем пользователю поиграть.
                 */
                if ($this->processingOldCommand() === false) {
                    $this->tts = "Упс...\nЯ тебя не понимаю...\nСкажи, ты готов поиграть в игру \"Я никогда не\"";
                    $this->card->title = "Скажи, ты готов поиграть в игру \"Я никогда не\"";
                    $this->card->add('997614/743bc730adb717da13d7', 'Да начинаем игру', '', 'Да начинаем игру');
                    $this->card->add('213044/31a85be9652f407e4af3', 'Нет не готов', '', 'Нет не готов');
                    $this->card->add('1030494/a4da1e3a4eb517387652', 'Выбрать режим игры', '', 'Выбрать режим игры');
                    $this->userData['step'] = 1;
                    $this->thisIntentName = 'game';
                }
                break;
        }
    }
}
